@@ -4,7 +4,7 @@ use warnings;
 use strict;
 use 5.008003;
 
-our $VERSION = '1.205';
+our $VERSION = '1.206';
 use Exporter 'import';
 our @EXPORT_OK = qw( choose );
 
@@ -53,38 +53,48 @@ sub DESTROY {
 }
 
 
-sub __undef_to_defaults {
+sub __defaults {
     my ( $self ) = @_;
     my $prompt = defined $self->{wantarray} ? 'Your choice:' : 'Close with ENTER';
-    $self->{prompt}       = $prompt      if ! defined $self->{prompt};
-    $self->{beep}         = 0            if ! defined $self->{beep};
-    $self->{clear_screen} = 0            if ! defined $self->{clear_screen};
-    #$self->{default}     = undef        if ! defined $self->{default};
-    $self->{empty}        = '<empty>'    if ! defined $self->{empty};
-    $self->{hide_cursor}  = 1            if ! defined $self->{hide_cursor};
-    $self->{index}        = 0            if ! defined $self->{index};
-    $self->{justify}      = 0            if ! defined $self->{justify};
-    $self->{keep}         = 5            if ! defined $self->{keep};
-    $self->{layout}       = 1            if ! defined $self->{layout};
-    #$self->{lf}          = undef        if ! defined $self->{lf};
-    #$self->{ll}          = undef        if ! defined $self->{ll};
-    #$self->{mark}        = undef        if ! defined $self->{mark};
-    #$self->{max_height}  = undef        if ! defined $self->{max_height};
-    #$self->{max_width}   = undef        if ! defined $self->{max_width};
-    $self->{mouse}        = 0            if ! defined $self->{mouse};
-    #$self->{no_spacebar} = undef        if ! defined $self->{no_spacebar};
-    $self->{order}        = 1            if ! defined $self->{order};
-    $self->{pad}          = 2            if ! defined $self->{pad};
-    $self->{pad_one_row}  = $self->{pad} if ! defined $self->{pad_one_row};
-    $self->{page}         = 1            if ! defined $self->{page};
-    $self->{undef}        = '<undef>'    if ! defined $self->{undef};
+    return {
+        prompt       => $prompt,
+        beep         => 0,
+        clear_screen => 0,
+        #default     => undef,
+        empty        => '<empty>',
+        hide_cursor  => 1,
+        index        => 0,
+        justify      => 0,
+        keep         => 5,
+        layout       => 1,
+        #lf          => undef,
+        #ll          => undef,
+        #mark        => undef,
+        #max_height  => undef,
+        #max_width   => undef,
+        mouse        => 0,
+        #no_spacebar => undef,
+        order        => 1,
+        pad          => 2,
+        #pad_one_row => pad,
+        page         => 1,
+        undef        => '<undef>',
+    };
 }
 
 
-sub __validate_and_add_options {
-    my ( $self, $opt ) = @_;
-    return if ! defined $opt;
-    my $valid = {
+sub __undef_to_defaults {
+    my ( $self ) = @_;
+    my $defaults = $self->__defaults();
+    for my $option ( keys %$defaults ) {
+        $self->{$option} = $defaults->{$option} if ! defined $self->{$option};
+    }
+    $self->{pad_one_row}  = $self->{pad} if ! defined $self->{pad_one_row};
+}
+
+
+sub __valid_options {
+    return {
         beep            => '[ 0 1 ]',
         clear_screen    => '[ 0 1 ]',
         default         => '[ 0-9 ]+',
@@ -108,36 +118,44 @@ sub __validate_and_add_options {
         prompt          => '',
         undef           => '',
     };
+};
+
+
+sub __validate_and_add_options {
+    my ( $self, $opt ) = @_;
+    return if ! defined $opt;
+    my $valid = $self->__valid_options();
     my $sub =  ( caller( 1 ) )[3];
-    $sub =~ s/^.+::([^:]+)\z/$1/;
+    $sub =~ s/^.+::(?:__)?([^:]+)\z/$1/;
+    $sub .= ':';
     for my $key ( keys %$opt ) {
         if ( ! exists $valid->{$key} ) {
-            croak "$sub: '$key' is not a valid option name";
+            croak "$sub '$key' is not a valid option name";
         }
         next if ! defined $opt->{$key};
         if ( $key eq 'lf' ) {
             if ( ! ( ref $opt->{$key} eq 'ARRAY' && @{$opt->{$key}} <= 2 ) ) {
-                croak "$sub: option '$key' : the passed value has to be an ARRAY reference.";
+                croak "$sub $key => the passed value has to be an ARRAY reference.";
             }
             no warnings 'uninitialized';
             for ( @{$opt->{$key}} ) {
-                /^[0-9]+\z/ or croak "$sub: option '$key' : $_ is an invalid array element";
+                /^[0-9]+\z/ or croak "$sub $key => $_ is an invalid array element";
             }
         }
         elsif ( $key eq 'no_spacebar' || $key eq 'mark' ) {
             if ( ref $opt->{$key} ne 'ARRAY' ) {
-                croak "$sub: option '$key' : the passed value has to be an ARRAY reference.";
+                croak "$sub $key => the passed value has to be an ARRAY reference.";
             }
             no warnings 'uninitialized';
             for ( @{$opt->{$key}} ) {
-                /^[0-9]+\z/ or croak "$sub: option '$key' : $_ is an invalid array element";
+                /^[0-9]+\z/ or croak "$sub $key => $_ is an invalid array element";
             }
         }
         elsif ( $valid->{$key} eq '' && ref $opt->{$key} ne '' ) {
-            croak "$sub: option '$key' : references are not valid values.";
+            croak "$sub $key => references are not valid values.";
         }
         elsif ( length $valid->{$key} && $opt->{$key} !~ m/^$valid->{$key}\z/x ) {
-            croak "$sub: option '$key' : '$opt->{$key}' is not a valid value.";
+            croak "$sub $key => '$opt->{$key}' is not a valid value.";
         }
         $self->{$key} = $opt->{$key};
     }
@@ -657,12 +675,12 @@ sub __copy_orig_list {
 
 sub __length_longest {
     my ( $self ) = @_;
+    my $list = $self->{list};
     if ( $self->{ll} ) {
         $self->{length_longest} = $self->{ll};
-        $self->{length} = [];
+        $self->{length} = [ ( $self->{length_longest} ) x @$list ];
     }
     else {
-        my $list = $self->{list};
         my $len = [];
         my $longest = 0;
         for my $i ( 0 .. $#$list ) {
@@ -793,18 +811,8 @@ sub __size_and_layout {
         $self->{rc2idx}[0] = [ 0 .. $#{$self->{list}} ];
     }
     elsif ( $self->{layout} == 3 ) {
-        if ( $self->{length_longest} <= $self->{avail_width} ) {
-            for my $idx ( 0 .. $#{$self->{list}} ) {
-                $self->{rc2idx}[$idx][0] = $idx;
-            }
-        }
-        else {
-            for my $idx ( 0 .. $#{$self->{list}} ) {
-                if ( $self->__print_columns( $self->{list}[$idx] ) > $self->{avail_width} ) {
-                    $self->{list}[$idx] = $self->__unicode_trim( $self->{list}[$idx], $self->{avail_width} - 3 ) . '...';
-                }
-                $self->{rc2idx}[$idx][0] = $idx;
-            }
+        for my $idx ( 0 .. $#{$self->{list}} ) {
+            $self->{rc2idx}[$idx][0] = $idx;
         }
     }
     else {
@@ -866,50 +874,6 @@ sub __print_columns {
 }
 
 
-sub __unicode_trim {
-    my ( $self, $str, $len ) = @_;
-    return '' if $len <= 0; #
-    my $gcs = Unicode::GCString->new( $str );
-    my $pos = $gcs->pos;
-    $gcs->pos( 0 );
-    my $cols = 0;
-    my $gc;
-    while ( defined( $gc = $gcs->next ) ) {
-        if ( $len < ( $cols += $gc->columns ) ) {
-            my $ret = $gcs->substr( 0, $gcs->pos - 1 );
-            $gcs->pos( $pos );
-            return $ret->as_string;
-        }
-    }
-    $gcs->pos( $pos );
-    return $gcs->as_string;
-}
-
-
-sub __prepare_page_number {
-    my ( $self ) = @_;
-    $self->{pp} = int( $#{$self->{rc2idx}} / ( $self->{avail_height} + $self->{tail} ) ) + 1;
-    if ( $self->{pp} > 1 ) {
-        $self->{pp} = int( $#{$self->{rc2idx}} / $self->{avail_height} ) + 1;
-        $self->{width_pp} = length $self->{pp};
-        $self->{pp_printf_fmt} = "--- Page %0*d/%d ---";
-        $self->{pp_printf_type} = 0;
-        if ( length sprintf( $self->{pp_printf_fmt}, $self->{width_pp}, $self->{pp}, $self->{pp} ) > $self->{avail_width} ) {
-            $self->{pp_printf_fmt} = "%0*d/%d";
-            if ( length sprintf( $self->{pp_printf_fmt}, $self->{width_pp}, $self->{pp}, $self->{pp} ) > $self->{avail_width} ) {
-                $self->{width_pp} = $self->{avail_width} if $self->{width_pp} > $self->{avail_width};
-                $self->{pp_printf_fmt} = "%0*.*s";
-                $self->{pp_printf_type} = 1;
-            }
-        }
-    }
-    else {
-        $self->{avail_height} += $self->{tail};
-        $self->{tail} = 0;
-    }
-}
-
-
 sub __set_default_cell {
     my ( $self ) = @_;
     $self->{tmp_pos} = [ 0, 0 ];
@@ -956,20 +920,57 @@ sub __goto {
 }
 
 
+sub __prepare_page_number {
+    my ( $self ) = @_;
+    $self->{pp} = int( $#{$self->{rc2idx}} / ( $self->{avail_height} + $self->{tail} ) ) + 1;
+    if ( $self->{pp} > 1 ) {
+        $self->{pp} = int( $#{$self->{rc2idx}} / $self->{avail_height} ) + 1;
+        $self->{width_pp} = length $self->{pp};
+        $self->{footer_fmt_short} = 0;
+        $self->{footer_fmt} = "--- Page %0*d/%d ---";
+        my $len_footer = length sprintf $self->{footer_fmt},
+                                        $self->{width_pp}, $self->{pp},
+                                        $self->{pp};
+        if ( $len_footer > $self->{avail_width} ) {
+            $self->{footer_fmt} = "%0*d/%d";
+            my $len_footer = length sprintf $self->{footer_fmt},
+                                            $self->{width_pp}, $self->{pp},
+                                            $self->{pp};
+            if (  $len_footer > $self->{avail_width} ) {
+                if ( $self->{width_pp} > $self->{avail_width} ) {
+                    $self->{width_pp} = $self->{avail_width};
+                }
+                $self->{footer_fmt_short} = "%0*.*s";
+            }
+        }
+    }
+    else {
+        $self->{avail_height} += $self->{tail};
+        $self->{tail} = 0;
+    }
+}
+
+
 sub __wr_screen {
     my ( $self ) = @_;
     $self->__goto( 0, 0 );
     $self->{plugin}->__clear_to_end_of_screen();
     if ( $self->{page} && $self->{pp} > 1 ) {
         $self->__goto( $self->{avail_height_idx} + $self->{tail}, 0 );
-        if ( $self->{pp_printf_type} == 0 ) {
-            printf $self->{pp_printf_fmt}, $self->{width_pp}, int( $self->{row_on_top} / $self->{avail_height} ) + 1, $self->{pp};
-            $self->{i_col} += length sprintf $self->{pp_printf_fmt}, $self->{width_pp}, int( $self->{row_on_top} / $self->{avail_height} ) + 1, $self->{pp};
+        my $page_number;
+        if ( $self->{footer_fmt_short} ) {
+            $page_number = sprintf $self->{footer_fmt_short},
+                                   $self->{width_pp}, $self->{width_pp},
+                                   int( $self->{row_on_top} / $self->{avail_height} ) + 1;
         }
-        elsif ( $self->{pp_printf_type} == 1 ) {
-            printf $self->{pp_printf_fmt}, $self->{width_pp}, $self->{width_pp}, int( $self->{row_on_top} / $self->{avail_height} ) + 1;
-            $self->{i_col} += length sprintf $self->{pp_printf_fmt}, $self->{width_pp}, $self->{width_pp}, int( $self->{row_on_top} / $self->{avail_height} ) + 1;
+        else {
+            $page_number = sprintf $self->{footer_fmt},
+                                   $self->{width_pp},
+                                   int( $self->{row_on_top} / $self->{avail_height} ) + 1,
+                                   $self->{pp};
         }
+        print $page_number;
+        $self->{i_col} += length $page_number;
      }
     for my $row ( $self->{p_begin} .. $self->{p_end} ) {
         for my $col ( 0 .. $#{$self->{rc2idx}[$row]} ) {
@@ -982,37 +983,64 @@ sub __wr_screen {
 
 sub __wr_cell {
     my( $self, $row, $col ) = @_;
+    my $is_current_pos = $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
+    my $idx = $self->{rc2idx}[$row][$col];
     if ( $#{$self->{rc2idx}} == 0 ) {
         my $lngth = 0;
         if ( $col > 0 ) {
             for my $cl ( 0 .. $col - 1 ) {
-                $lngth += $self->__print_columns( $self->{list}[$self->{rc2idx}[$row][$cl]] );
+                my $i = $self->{rc2idx}[$row][$cl];
+                $lngth += $self->__print_columns( $self->{list}[$i] );
                 $lngth += $self->{pad_one_row};
             }
         }
         $self->__goto( $row - $self->{row_on_top}, $lngth );
         $self->{plugin}->__bold_underline() if $self->{marked}[$row][$col];
-        $self->{plugin}->__reverse()        if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
-        print $self->{list}[$self->{rc2idx}[$row][$col]];
-        $self->{i_col} += $self->__print_columns( $self->{list}[$self->{rc2idx}[$row][$col]] );
+        $self->{plugin}->__reverse()        if $is_current_pos;
+        print $self->{list}[$idx];
+        $self->{i_col} += $self->__print_columns( $self->{list}[$idx] );
     }
     else {
         $self->__goto( $row - $self->{row_on_top}, $col * $self->{col_width} );
         $self->{plugin}->__bold_underline() if $self->{marked}[$row][$col];
-        $self->{plugin}->__reverse()        if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
-        print $self->__unicode_sprintf( $self->{rc2idx}[$row][$col] );
-        $self->{i_col} += $self->{length_longest};
+        $self->{plugin}->__reverse()        if $is_current_pos;
+        print $self->__unicode_sprintf( $idx );
+        $self->{i_col} += $self->{avail_col_width};
     }
-    $self->{plugin}->__reset() if $self->{marked}[$row][$col] || $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
+    $self->{plugin}->__reset() if $self->{marked}[$row][$col] || $is_current_pos;
+}
+
+
+sub __unicode_trim {
+    my ( $self, $str, $len ) = @_;
+    my $gcs = Unicode::GCString->new( $str );
+    my $pos = $gcs->pos;
+    $gcs->pos( 0 );
+    my $cols = 0;
+    my $gc;
+    while ( defined( $gc = $gcs->next ) ) {
+        if ( $len < ( $cols += $gc->columns ) ) {
+            my $ret = $gcs->substr( 0, $gcs->pos - 1 );
+            $gcs->pos( $pos );
+            return $ret->as_string;
+        }
+    }
+    $gcs->pos( $pos );
+    return $gcs->as_string;
 }
 
 
 sub __unicode_sprintf {
     my ( $self, $idx ) = @_;
     my $unicode;
-    my $str_length = defined $self->{length}[$idx] ? $self->{length}[$idx] : $self->{length_longest};
+    my $str_length = $self->{length}[$idx];
     if ( $str_length > $self->{avail_col_width} ) {
-        $unicode = $self->__unicode_trim( $self->{list}[$idx], $self->{avail_col_width} );
+        if ( $self->{avail_col_width} > 3 ) {
+            $unicode = $self->__unicode_trim( $self->{list}[$idx], $self->{avail_col_width} - 3 ) . '...';
+        }
+        else {
+            $unicode = $self->__unicode_trim( $self->{list}[$idx], $self->{avail_col_width} );
+        }
     }
     elsif ( $str_length < $self->{avail_col_width} ) {
         if ( $self->{justify} == 0 ) {
@@ -1053,7 +1081,8 @@ sub __mouse_info_to_key {
         if ( $row == $mouse_row ) {
             my $end_last_col = 0;
             COL: for my $col ( 0 .. $#{$self->{rc2idx}[$row]} ) {
-                my $end_this_col = $end_last_col + $self->__print_columns( $self->{list}[$self->{rc2idx}[$row][$col]] ) + $self->{pad_one_row};
+                my $idx = $self->{rc2idx}[$row][$col];
+                my $end_this_col = $end_last_col + $self->__print_columns( $self->{list}[$idx] ) + $self->{pad_one_row};
                 if ( $col == 0 ) {
                     $end_this_col -= int( $self->{pad_one_row} / 2 );
                 }
@@ -1130,7 +1159,7 @@ Term::Choose - Choose items from a list interactively.
 
 =head1 VERSION
 
-Version 1.205
+Version 1.206
 
 =cut
 
@@ -1697,49 +1726,23 @@ It is required a terminal that uses a monospaced font which supports the printed
 
 =head2 Escape sequences
 
-The following ANSI escape sequences are used:
+It is required a terminal that supports ANSI escape sequences.
 
-    "\e[A"      Cursor Up
-
-    "\e[C"      Cursor Forward
-
-    "\e[D"      Cursor Back
-
-    "\e[0J"     Clear to End of Screen (Erase Data)
-
-    "\e[0m"     Normal/Reset
-
-    "\e[1m"     Bold
-
-    "\e[4m"     Underline
-
-    "\e[7m"     Inverse
-
-If the option "hide_cursor" is enabled:
+If the option "hide_cursor" is enabled, it is also required the support for the following escape sequences:
 
     "\e[?25l"   Hide Cursor
 
     "\e[?25h"   Show Cursor
 
-If the option "clear_screen" is enabled:
-
-    "\e[2J"     Clear Screen (Erase Data)
-
-    "\e[1;1H"   Go to Top Left (Cursor Position)
-
-If the OS is MSWin32 L<Win32::Console> is used, to emulate these escape sequences.
-
 If a I<mouse> mode is enabled
 
-    "\e[6n"    Get Cursor Position (Device Status Report)
-
     "\e[?1003h", "\e[?1005h", "\e[?1006h"   Enable Mouse Tracking
 
     "\e[?1003l", "\e[?1005l", "\e[?1006l"   Disable Mouse Tracking
 
 are used to enable/disable the different I<mouse> modes.
 
-To read key and mouse events with an MSWin32 OS L<Win32::Console> is used instead.
+If the OS is MSWin32 L<Win32::Console> is used, to emulate the behavior of the escape sequences.
 
 =head1 SUPPORT
 
